library flutter_firebase_basic_tools;

import 'package:flutter/material.dart';

enum FormFieldValidatorsType {
  email,
  easyPassword,
  mediumPassword,
  hardPassword,
  firstname,
  lastname,
  custom,
}

class FormFieldValidators {
  const FormFieldValidators({
    @required this.value,
    @required this.type,
    this.customMessage,
    this.customPattern,
  });

  final String value;
  final FormFieldValidatorsType type;
  final String customMessage;
  final Pattern customPattern;

  String validate() {
    Pattern pattern;
    Pattern message;

    switch (type) {
      case FormFieldValidatorsType.email:
        {
          pattern =
          r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$';
          message = 'Email format is invalid';
        }
        break;

      // Minimum eight characters, at least one letter and one number

      case FormFieldValidatorsType.easyPassword:
        {
          pattern =
          r"^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d]{8,}$";
          message = 'Enter a valid password';
        }
        break;

      // Minimum eight characters, at least one uppercase letter, one lowercase letter and one number

      case FormFieldValidatorsType.mediumPassword:
        {
          pattern =
          r"^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)[a-zA-Z\d]{8,}$";
          message = 'Enter a valid password';
        }
        break;

      // Minimum eight characters, at least one uppercase letter, one lowercase letter, one number and one special character

      case FormFieldValidatorsType.hardPassword:
        {
          pattern =
          r"^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&.])[A-Za-z\d@$!%*?&.]{8,100}$";
          message = 'Enter a valid password';
        }
        break;

      case FormFieldValidatorsType.firstname:
        {
          pattern = r"^[a-zA-Z]{2,25}$";
          message = "Enter a valid first name";
        }
        break;

      case FormFieldValidatorsType.lastname:
        {
          pattern = r"^[a-zA-Z]{2,25}$";
          message = "Enter a valid last name";
        }
        break;

      case FormFieldValidatorsType.custom:
        {
          if (customMessage != null && customPattern != null) {
            message = customMessage;
            pattern = customPattern;
          } else {
            assert(
            customMessage != null && customPattern != null,
            "Custom validator require a custom message and a custom pattern",
            );
          }
        }
        break;
    }

    RegExp regex = new RegExp(pattern);
    if (!regex.hasMatch(value)) {
      return message;
    } else {
      return null;
    }
  }
}